//+-------------------------------------------------------------------------------------------------+
//|                                                                            CDownLoadHistory.mqh |
//|                                                                                   2011, etrader |
//|                                                                            http://efftrading.ru |
//+-------------------------------------------------------------------------------------------------+
#property copyright "2011, etrader"
#property link      "http://efftrading.ru"

#include "ClassProgressBar.mqh" // http://www.mql5.com/ru/articles/17
//+------------------------------------------------------------------+
//| history download modes                                           |
//+------------------------------------------------------------------+
enum ENUM_DOWNLOADHISTORYMODE         // history download mode
  {
   DOWNLOADHISTORYMODE_CURRENTSYMBOL, // Current symbol
   DOWNLOADHISTORYMODE_ALLSYMBOLS     // All symbols from Market Watch window
  };
//+-------------------------------------------------------------------------------------------------+
//| Class, used for downloading of historical data                                                  |
//+-------------------------------------------------------------------------------------------------+
class CDownLoadHistory
  {
private:
   int               lasterror;         // Last error code
   bool              isvisualmode;      // Visual mode used
   string            symbol;            // Symbol (or NULL for all symbols from Market Watch window)
   CProgressBar      progresscurrent;   // Progress bar for current symbol
   CProgressBar      progressall;       // Progress bar for all symbols

public:
//---- Object init
   void              Create( string symbolpass=NULL, bool isvisualmodepass=false );
//--- Error description
   string            ErrorDescription( int nerr );
//---- Download
   int               Execute();
//---- Download data for symbol
   int               ExecuteOneSymbol( string symbol);                               
//---- Last error code
   int  LastError(){return( lasterror );};                              
//---- Load history for symbol (start_date)
   int               CheckLoadHistory(string symbol,ENUM_TIMEFRAMES period,datetime start_date);
  };
//+-------------------------------------------------------------------------------------------------+
//| Loads the history for the specified symbol                                                      |
//| input:  sym - symbol                                                                            |
//| output: >=0 - if successful, <0 - if error (error code)                                         |
//+-------------------------------------------------------------------------------------------------+
int CDownLoadHistory::ExecuteOneSymbol(string sym)
  {

   MqlDateTime fromtime;
   TimeLocal(fromtime);

//loop years, starting from current to the year of the server's first history date 
   datetime begserver=(datetime)SeriesInfoInteger(sym,PERIOD_D1,SERIES_SERVER_FIRSTDATE);
   MqlDateTime tm;
   TimeToStruct(begserver,tm);
   int res=0;
   if(isvisualmode)
     {
      progresscurrent.Create(0,"Loading",0,150,20);   // Progress bar for the current symbol
      progresscurrent.Text(sym+" Completed : ");
      progresscurrent.Value( (int)0.00001 );
     }

//loop years
   for(int i=fromtime.year; i>=tm.year; i--)
     {
      string sd=IntegerToString(i)+".01.01";
      // load history (sd date)
      res=CheckLoadHistory(sym,PERIOD_D1,StringToTime(sd));
      if(res<0)
        {
         lasterror=res;
         return(res);
         break;
        }
      if(isvisualmode)progresscurrent.Value((int)((0.+fromtime.year-i)/(-tm.year+fromtime.year)*100));
     }

   if(isvisualmode)
     {
      progresscurrent.Delete();
     }
   return(res);
  }
//+-------------------------------------------------------------------------------------------------+
//| Create method                                                                                   |
//| Input: symbolpass - symbol, if NULL, all symbols from Market Watch window                       |
//|       isvisualmodepass - visual mode                                                            |
//| Output: None                                                                                    |
//+-------------------------------------------------------------------------------------------------+
void CDownLoadHistory::Create(string symbolpass=NULL,bool isvisualmodepass=false)
  {
   symbol=symbolpass;
   isvisualmode=isvisualmodepass;
   lasterror=0;
  }
//+-------------------------------------------------------------------------------------------------+
//| Returns error description                                                                       |
//| Input:  nerr - error code                                                                       |
//| Output: Text description of error                                                               |                    
//+-------------------------------------------------------------------------------------------------+
string CDownLoadHistory::ErrorDescription(int nerr)
  {
   if(nerr>=0)return("No errors");
   if(nerr == -1 )return("Unknown symbol");
   if(nerr == -2 )return("Requested bars exceeds the max bars on the chart");
   if(nerr == -3 )return("Interrupted by user");
   if(nerr == -4 )return("The indicator should not load its own data");
   if(nerr == -5 )return("Download failed");
   return("Unknown error");
  }
//+-------------------------------------------------------------------------------------------------+
//| Download history                                                                                |
//| Input: None                                                                                     |       
//| Output: >=0 - if successful, <0 - if error (error code)                                         |
//+-------------------------------------------------------------------------------------------------+
int CDownLoadHistory::Execute()
  {

   if(symbol!=NULL)return(ExecuteOneSymbol(symbol));  // For one symbol

   int total=SymbolsTotal(true);
   int res=0;
   if(isvisualmode)
     {
      progressall.Create(0,"DownloadTotal",0,150,40);   // Progress bar
      progressall.Text("Total ");
     }
   for(int i=0; i<total; i++)
     {
      string cursym=SymbolName(i,true);
      if(isvisualmode)
        {
        }
      res=ExecuteOneSymbol(cursym);
      if(res<0)
        {
         lasterror=res;
         return(res);
        }
      if(isvisualmode)
        {
         int prc=(int)(1.*i/total*100);
         progressall.Value(prc);
        }
     }
   return(res);
  }
//+------------------------------------------------------------------+
//| Checks presence of the history starting from start_date,         |
//| if the history is not exist, it will try to download it          |
//| INPUT : symbol                                                   |
//|         period - timeframe                                       |
//|         start_date - starting date                               |
//| OUTPUT: Result code                                              |
//|         see http://www.mql5.com/en/docs/series/timeseries_access |
//| Note  : none                                                     |
//+------------------------------------------------------------------+ 
int CDownLoadHistory::CheckLoadHistory(string symbol,ENUM_TIMEFRAMES period,datetime start_date)
  {
   datetime first_date=0;
   datetime times[100];
//--- check symbol & period
   if(symbol==NULL || symbol=="") symbol=Symbol();
   if(period==PERIOD_CURRENT)     period=Period();
//--- check if symbol is selected in the MarketWatch
   if(!SymbolInfoInteger(symbol,SYMBOL_SELECT))
     {
      if(GetLastError()==ERR_MARKET_UNKNOWN_SYMBOL) return(-1);
      SymbolSelect(symbol,true);
     }
//--- check if data is present
   SeriesInfoInteger(symbol,period,SERIES_FIRSTDATE,first_date);
   if(first_date>0 && first_date<=start_date) return(1);
//--- don't ask for load of its own data if it is an indicator
   if(MQL5InfoInteger(MQL5_PROGRAM_TYPE)==PROGRAM_INDICATOR && Period()==period && Symbol()==symbol)
      return(-4);
//--- second attempt
   if(SeriesInfoInteger(symbol,PERIOD_M1,SERIES_TERMINAL_FIRSTDATE,first_date))
     {
      //--- there is loaded data to build timeseries
      if(first_date>0)
        {
         //--- force timeseries build
         CopyTime(symbol,period,first_date+PeriodSeconds(period),1,times);
         //--- check date
         if(SeriesInfoInteger(symbol,period,SERIES_FIRSTDATE,first_date))
            if(first_date>0 && first_date<=start_date) return(2);
        }
     }
//--- max bars in chart from terminal options
   int max_bars=TerminalInfoInteger(TERMINAL_MAXBARS);
//--- load symbol history info
   datetime first_server_date=0;
   while(!SeriesInfoInteger(symbol,PERIOD_M1,SERIES_SERVER_FIRSTDATE,first_server_date) && !IsStopped())
      Sleep(5);
//--- fix start date for loading
   if(first_server_date>start_date) start_date=first_server_date;
   if(first_date>0 && first_date<first_server_date)
      Print("Warning: first server date",first_server_date,"for",symbol,"does not match to first series date",first_date);
//--- load data step by step

   int fail_cnt=0;
   while(!IsStopped())
     {
      //--- wait for timeseries build
      while(!SeriesInfoInteger(symbol,period,SERIES_SYNCRONIZED) && !IsStopped())
         Sleep(5);
      //--- ask for built bars
      int bars=Bars(symbol,period);
      if(bars>0)
        {
         if(bars>=max_bars) return(-2);
         //--- ask for first date
         if(SeriesInfoInteger(symbol,period,SERIES_FIRSTDATE,first_date))
            if(first_date>0 && first_date<=start_date) return(0);
        }
      //--- copying of next part forces data loading
      int copied=CopyTime(symbol,period,bars,100,times);
      if(copied>0)
        {
         //--- check for data
         if(times[0]<=start_date)  return(0);
         if(bars+copied>=max_bars) return(-2);
         fail_cnt=0;
        }
      else
        {
         //--- no more than 100 failed attempts
         fail_cnt++;
         if(fail_cnt>=100) return(-5);
         Sleep(10);
        }
     }
//--- stopped
   return(-3);
  }
//+------------------------------------------------------------------+
